
#include "TS.h"
#include "Screen.h"
#include "Game.h"
#include "string.h"
#include "IO.h"
#include "Well.h"
#include "stdlib.h"

#define repeat_pause 4

u16 score = 0;

static u8  wait_scroll  = 0;
static u8  wait_move    = 0;


// ctrl
static u8  wait_up      = 0;
static u8  rnd;
static u8  next;
static t_shape_pos shape_pos;
static u8 remove_fase = 0;
static u8 remove_lines[4];

const static u8 colors[7] = {1,2,3,4,5,4,6};

u8  GetRnd(u8 min, u8 max)
{
    while (1)
    {
        __asm
            push af
            ld   a,r
            and  a, #0x0F
            ld   (_rnd),a
            pop  af
        __endasm;

        if (rnd >= min && rnd <= max)
            break;
    }

    return rnd;
}

void AddScore(u8 value)
{
    score += value;
}

void DrawScore()
{
    u16 t = score;
    DrawDigit(34, 17, t % 10, T_SCORE);
    t/=10;
    DrawDigit(30, 17, t % 10, T_SCORE);
    t/=10;
    DrawDigit(26, 17, t % 10, T_SCORE);
    t/=10;
    DrawDigit(22, 17, t % 10, T_SCORE);
}

void DrawScene()
{
    u8 x,y;

    ClearScreen();
    //DIHALT(0xF5)
    // 
    for(y = pad_top; y <= pad_top + pad_height; y++)
    {
        tilemap[y][1][pad_left]  = T_BORDER;
        tilemap[y][1][pad_left + pad_width+1] = T_BORDER;
    }
    for(x = pad_left+1; x <= pad_left+pad_width;x++)
    {
        tilemap[pad_top + pad_height][1][x] = T_BORDER;
    }

    //  
    DrawRect(20, 15, 19, 9, T_BORDER);
    DrawScore();

    //  
    DrawRect(20, 1, 9, 7, T_BORDER);
}

void DrawFrame()
{
    u8 i, y,color;

    if (remove_fase == 0)
        Control();
    else
    {
        // animation
        if (remove_fase > 1)
        {
            color = (remove_fase & 2)? 0 : 7;

            for(i=0;i<4;i++)
            {
                if (remove_lines[i] == 0) break;
                memset(&well[remove_lines[i]][4], color, well_width);
            }
        }
        else
        {
            for(i=0;i<4;i++)
            {
                if (remove_lines[i] == 0) break;

                memset(&well[3][4], 0, well_width);
                for(y=remove_lines[i]+i; y>3;y--)
                    memcpy(&well[y][4], &well[y-1][4], well_width);
            }
        }
        remove_fase--;
    }
    ShowWell();
}

void NewShape()
{
    shape_pos.shape = next;
    shape_pos.rotate  = rand()%3;
    next = rand()%6;

    shape_pos.x = 9;
    shape_pos.y = 2;

    DrawFillRect(21,2,7,5,0);
    DrawTileShape(22,3, next, 1, colors[next]);

    DrawScore();
}

void RemoveFullLine()
{
    u8 x,y, i = 0;

    memset(&remove_lines, 0, sizeof(remove_lines));

    y = well_height+3;
    while(y>=shape_pos.y)
    {
        for(x=0;x<well_width;x++)
            if (well[y][x+4] == 0) goto l1;

        remove_lines[i++] = y;
        remove_fase = 20;
        l1: y--;
    }

    // update score
    score += i*i;
}

void GameOver()
{
    Reset();
}

void MoveDown()
{
    t_shape_pos new_pos;

    memcpy(&new_pos, &shape_pos, sizeof(shape_pos));
    new_pos.y++;
    // remove shape
    DrawShape(&shape_pos, 0);

    if (IsCollision(&new_pos))
    {
        DrawShape(&shape_pos, colors[shape_pos.shape]);

        if (new_pos.y < 4)
            GameOver();
        else
        {
            RemoveFullLine();
            NewShape();
        }
        return;
    }
    else
    {
        memcpy(&shape_pos, &new_pos, sizeof(shape_pos));
        DrawShape(&shape_pos, colors[shape_pos.shape]);
    }
}

void MoveLeft()
{
    t_shape_pos new_pos;
    memcpy(&new_pos, &shape_pos, sizeof(shape_pos));

    new_pos.x--;
    // remove shape
    DrawShape(&shape_pos, 0);

    if (!IsCollision(&new_pos))
        memcpy(&shape_pos, &new_pos, sizeof(shape_pos));

    DrawShape(&shape_pos, colors[shape_pos.shape]);
}

void MoveRight()
{
    t_shape_pos new_pos;
    memcpy(&new_pos, &shape_pos, sizeof(shape_pos));

    new_pos.x++;
    // remove shape
    DrawShape(&shape_pos, 0);

    if (!IsCollision(&new_pos))
        memcpy(&shape_pos, &new_pos, sizeof(shape_pos));

    DrawShape(&shape_pos, colors[shape_pos.shape]);
}

void Rotate()
{
    t_shape_pos new_pos;

    memcpy(&new_pos, &shape_pos, sizeof(shape_pos));
    new_pos.rotate = (new_pos.rotate+1) & 0x03;
    // remove shape
    DrawShape(&shape_pos, 0);

    if (!IsCollision(&new_pos))
        memcpy(&shape_pos, &new_pos, sizeof(shape_pos));

    DrawShape(&shape_pos, colors[shape_pos.shape]);
}

void Control()
{
    DrawShape(&shape_pos, colors[shape_pos.shape]);

    if (wait_scroll > 0)
        wait_scroll--;
    else
    {
        if (JOY_ISDOWN || CURSOR_ISDOWN)
            wait_scroll = 1;
        else
            wait_scroll = 30;

        MoveDown();
        return;
    }

    if (JOY_ISUP || CURSOR_ISUP)
    {
        if (wait_up >0)
            wait_up--;
        else
        {
            Rotate();
            wait_up = repeat_pause;
        }
    }
    else
        wait_up = 0;

    if (JOY_ISLEFT || JOY_ISRIGHT || CURSOR_ISLEFT || CURSOR_ISRIGHT)
    {
        if (wait_move >0)
            wait_move--;
        else
        {
            if (JOY_ISLEFT || CURSOR_ISLEFT) MoveLeft();
            if (JOY_ISRIGHT || CURSOR_ISRIGHT) MoveRight();
            wait_move = repeat_pause;
        }
    }
    else
        wait_move = 0;
}

void StageAnimation()
{
    u8 y;

    y = well_height+7;
    while (y>0)
    {
        memset(&well[y][0], 5, well_width+8);
        y--;
        HALT
        ShowWell();
    }

    y = 0;
    while (y<well_height+7)
    {
        memset(&well[y][0], 0, well_width+8);
        y++;
        HALT
        ShowWell();
    }

}

void Reset()
{
    next = rand()%6;
    remove_fase = 0;
    memset(&remove_lines, 0, 4);
    DrawScene();

    StageAnimation();
    WellClear();

    /*well[well_height+3][4] = 1;
    well[well_height+3][5] = 2;
    well[well_height+3][6] = 3;
    well[well_height+3][7] = 4;
    well[well_height+3][8] = 5;
    well[well_height+3][9] = 6;
    well[well_height+3][10] = 1;
    well[well_height+3][11] = 1;

    well[well_height+3][13] = 6;
    well[well_height+3][14] = 1;
    well[well_height+3][15] = 6;
    well[well_height+3][16] = 1;
    well[well_height+3][17] = 2;

    well[well_height+2][4] = 1;
    well[well_height+2][5] = 2;
    well[well_height+2][6] = 3;
    well[well_height+2][7] = 4;
    well[well_height+2][8] = 5;
    well[well_height+2][9] = 6;
    well[well_height+2][10] = 1;
    well[well_height+2][11] = 1;

    well[well_height+2][13] = 6;
    well[well_height+2][14] = 1;
    well[well_height+2][15] = 6;
    well[well_height+2][16] = 1;
    well[well_height+2][17] = 2;
    */

    NewShape();
    DrawFrame();
}

void Logo()
{
    u8 cnt = 100;
    u8 x,y;
    u8 t;

    TS_Border = 0xF7;

    WellClear();
    ClearScreen();
    DrawFillRect(0,0,40,25, 7);

    for(y=0;y<5;y++)
    for(x=0;x<24;x++)
    {
        t = logo_pattern[y][x];
        tilemap[y+10][1][x+5] = TILE_INFO(0,0,3,t);
    }

    while(cnt-- > 0)
    {
        t = cnt & 0x07;

        for(x=0;x<23;x++)
        {
            tilemap[17][1][x+5] = TILE_INFO(0,0,3,t);
            t = (t+1) & 0x07;
        }

        HALT;
    }

    TS_Border = 0xF0;
}

