
#include "Screen.h"
#include "string.h"
#include "Well.h"

const u8 pattern[8] = { 0xFE, 0x82, 0xBA, 0xBA,  0xBA, 0x82, 0xFE, 0x00};
const u8 digits[10][5][3] =
{
    { // 0
        {1,1,1},{1,0,1},{1,0,1},{1,0,1},{1,1,1},
    },
    { // 1
        {1,1,0},{0,1,0},{0,1,0},{0,1,0},{1,1,1},
    },
    { // 2
        {1,1,1},{0,0,1},{1,1,1},{1,0,0},{1,1,1},
    },
    { // 3
        {1,1,1},{0,0,1},{1,1,1},{0,0,1},{1,1,1},
    },
    { // 4
        {1,0,1},{1,0,1},{1,1,1},{0,0,1},{0,0,1},
    },
    { // 5
        {1,1,1},{1,0,0},{1,1,1},{0,0,1},{1,1,1},
    },
    { // 6
        {1,1,1},{1,0,0},{1,1,1},{1,0,1},{1,1,1},
    },
    { // 7
        {1,1,1},{0,0,1},{0,0,1},{0,0,1},{0,0,1},
    },
    { // 8
        {1,1,1},{1,0,1},{1,1,1},{1,0,1},{1,1,1},
    },
    { // 9
        {1,1,1},{1,0,1},{1,1,1},{0,0,1},{1,1,1},
    }
};
const u8 logo_pattern[5][24] =
{
    { 1,1,1,7,2,2,2,7,1,1,1,7,3,3,3,7,4,7,5,5,5,7,4,7 },
    { 7,1,7,7,2,7,7,7,7,1,7,7,3,7,3,7,4,7,5,7,7,7,4,7 },
    { 7,1,7,7,2,2,2,7,7,1,7,7,3,3,3,7,4,7,5,5,5,7,4,7 },
    { 7,1,7,7,2,7,7,7,7,1,7,7,3,3,7,7,4,7,7,7,5,7,7,7 },
    { 7,1,7,7,2,2,2,7,7,1,7,7,3,7,3,7,4,7,5,5,5,7,4,7 }
};

void CreateTiles()
{
    u8 color,x,y,b;
    u8 ch;

    TS_Page3   = 0x20;

    for(color=0; color<8; color++)
        for(y=0; y<8; y++)
        {
            if (color == 0)
                b = 0;
            else
                b = pattern[y];

            for(x=0;x<4;x++)
            {
                ch = (b & 0x02) ? color : 8;
                ch |= (((b & 0x01) ? color : 8) << 4);

                tiles[y][color][x] = ch;
                b >>=2;
            }
        }

    TS_Page3   = 0x07;
}


void SetVMode()
{
    TS_VConfig = TS_VM_256C | TS_RRES_320X200 | TS_NOGFX;
    TS_Config  = TS_T1_EN | TS_T1_Z_EN;
    TS_PalSel  = 3 << 6; // T1_PAL[7:6]=11
    TS_T1GPage = 0x20;
    TS_TMPage  = 0x07;
}

void ClearScreen()
{
    //u8 x,y;

    memset(&tilemap, 0, sizeof(tilemap));

    /*for(x=0;x<64;x++)
        for(y=0;y<64;y++)
            tilemap[y][1][x] = TILE_INFO(0,0,3,0);*/
}

void DrawDigit(u8 pos_x, u8 pos_y, u8 value, u8 color)
{
    u8  x,y;
    u16 c;

    for(x=0;x<3;x++)
    for(y=0;y<5;y++)
    {
        c = T_NONE;
        if (digits[value][y][x]) c = TILE_INFO(0,0,3,color);
        tilemap[pos_y+y][1][pos_x+x] = c;
    }
}

void DrawRect(u8 left, u8 top, u8 width, u8 height, u8 color)
{
    u8 i;

    for(i = left; i <= left+width-1; i++)
    {
        tilemap[top][1][i] = TILE_INFO(0,0,3,color);
        tilemap[top+height-1][1][i] = TILE_INFO(0,0,3,color);
    }

    for(i = top+1; i <= top+height-1; i++)
    {
        tilemap[i][1][left] = TILE_INFO(0,0,3,color);
        tilemap[i][1][left+width-1] = TILE_INFO(0,0,3,color);
    }
}

void DrawFillRect(u8 left, u8 top, u8 width, u8 height, u8 color)
{
    u8 x,y;

    for(x = left; x <= left+width-1; x++)
        for(y = top; y <= top+height-1; y++)
            tilemap[y][1][x] = TILE_INFO(0,0,3,color);
}

void DrawTileShape(u8 pos_x, u8 pos_y, u8 shape, u8 rotate, u8 color)
{
    u8 x,y;
    u8 c;
    const u8* img = &shape_pattern[shape][rotate][0];

    for(y=0;y<4;y++)
    {
        c = img[y];
        for(x=0;x<4;x++)
        {
            if (c & 0x08)
                tilemap[pos_y+y][1][pos_x+x] = TILE_INFO(0,0,3,color);
            c <<=1;
        }
    }

}

