
#include "TS.h"

static u8 irqTab[257+6];  //  + 6     

static void** (dma_handler);
static void** (line_handler);
static void** (frame_handler);
static u8 im2addr;

static void DummyIRQ() __interrupt __naked
{
    __asm
        reti
    __endasm;
}

void InitTS()
{
    u16 addr = (u16)&irqTab;

    TS_T0XOffsL = 0;
    TS_T0XOffsH = 0;
    TS_T0YOffsL = 0;
    TS_T0YOffsH = 0;

    TS_T1XOffsL = 0;
    TS_T1XOffsH = 0;
    TS_T1YOffsL = 0;
    TS_T1YOffsH = 0;

    im2addr = addr >> 8;
    if ((addr & 0xFF) > 0xFB)
        im2addr++;

    dma_handler =   (void*())((u16)im2addr << 8 | 0xFB);
    line_handler =  (void*())((u16)im2addr << 8 | 0xFD);
    frame_handler = (void*())((u16)im2addr << 8 | 0xFF);

    *dma_handler =   &DummyIRQ;
    *line_handler =  &DummyIRQ;
    *frame_handler = &DummyIRQ;

    __asm
        di
        im   2
        push af
        ld   a, (_im2addr)
        ld   i,a
        pop  af
        ei
    __endasm;
}

void InitIRQ(void(*dma_irq), void(*line_irq), void(*frame_irq))
{
    u8 mask = 0x07;

    DI

    *dma_handler = dma_irq;
    *line_handler = line_irq;
    *frame_handler = frame_irq;

    if (dma_irq == nil)
    {
        mask &= TS_INTM_DMA ^ 0xFF;
        *dma_handler = &DummyIRQ;
    }


    if (line_irq == nil)
    {
        mask &= TS_INTM_LINE ^ 0xFF;
        *line_handler = &DummyIRQ;
    }

    if (frame_irq == nil)
    {
        mask &= TS_INTM_FRAME ^ 0xFF;
        *frame_handler = &DummyIRQ;
    }

    TS_INTMASK = mask;

    EI
}

