#ifndef TS_H_INCLUDED
#define TS_H_INCLUDED

typedef unsigned char u8;
typedef unsigned int  u16;

#define nil     0

#define TS_BASE 0xAF
#define TS_PORT(x) __sfr __banked __at((x<<8) | TS_BASE)

//TS_PORT(0xFF-0xFF)   TS_VConfig;
__sfr __banked __at(0x00AF)   TS_VConfig;
#define TS_RRES_256X192     0 << 6
#define TS_RRES_320X200     1 << 6
#define TS_RRES_320X240     2 << 6
#define TS_RRES_360X288     3 << 6
#define TS_NOGFX            1  << 5
#define TS_VM_ZX            0 << 0
#define TS_VM_16C           1 << 0
#define TS_VM_256C          2 << 0
#define TS_VM_TEXT          3 << 0

#define VMODE_ZX            TS_VM_ZX   | TS_RRES_256X192
#define VMODE_TEXT          TS_VM_TEXT | TS_RRES_320X200

#define VMODE_320X200X16    TS_VM_16C  | TS_RRES_320X200
#define VMODE_320X240X16    TS_VM_16C  | TS_RRES_320X240
#define VMODE_360X288X16    TS_VM_16C  | TS_RRES_360X288

#define VMODE_320X200X16    TS_VM_16C  | TS_RRES_320X200
#define VMODE_320X240X16    TS_VM_16C  | TS_RRES_320X240
#define VMODE_360X288X16    TS_VM_16C  | TS_RRES_360X288

TS_PORT(0x01)   TS_VPage;

TS_PORT(0x02)   TS_GXOffsL;
TS_PORT(0x03)   TS_GXOffsH;
TS_PORT(0x04)   TS_GYOffsL;
TS_PORT(0x05)   TS_GYOffsH;

TS_PORT(0x06)   TS_Config;
#define TS_S_EN             1 << 7
#define TS_T1_EN            1 << 6
#define TS_T0_EN            1 << 5
#define TS_T1_Z_EN          1 << 3
#define TS_T0_Z_EN          1 << 2

TS_PORT(0x07)   TS_PalSel;


TS_PORT(0x0F)   TS_Border;

TS_PORT(0x10)   TS_Page0;
TS_PORT(0x11)   TS_Page1;
TS_PORT(0x12)   TS_Page2;
TS_PORT(0x13)   TS_Page3;

TS_PORT(0x16)   TS_TMPage;
TS_PORT(0x17)   TS_T0GPage;
TS_PORT(0x18)   TS_T1GPage;

TS_PORT(0x2A)   TS_INTMASK;
#define TS_INTM_DMA         1 << 2
#define TS_INTM_LINE        1 << 1
#define TS_INTM_FRAME       1 << 0

TS_PORT(0x40)   TS_T0XOffsL;
TS_PORT(0x41)   TS_T0XOffsH;
TS_PORT(0x42)   TS_T0YOffsL;
TS_PORT(0x43)   TS_T0YOffsH;
TS_PORT(0x44)   TS_T1XOffsL;
TS_PORT(0x45)   TS_T1XOffsH;
TS_PORT(0x46)   TS_T1YOffsL;
TS_PORT(0x47)   TS_T1YOffsH;



#define EI          __asm ei __endasm;
#define DI          __asm di __endasm;
#define HALT        __asm halt __endasm;
#define DIHALT(x)   TS_Border = x; \
                    __asm \
                    di \
                    halt \
                    __endasm;

typedef union
{
    struct
    {
        u8 XF:1;
        u8 YF:1;
        u8 TPAL:2;
        u8 TNUM:12;
    };
    u16 u16;
} TileInfo;


#define BIT_L1(x,shift) ((x & 0x01) << shift)
#define BIT_L2(x,shift) ((x & 0x03) << shift)

#define TILE_INFO(XF,YF,TPAL,TNUM) (u16)(BIT_L1(XF,15) | BIT_L1(YF,14) | BIT_L2(TPAL, 12) | (TNUM & 0xFFF))


void InitTS();
void InitIRQ(void(*dma_irq), void(*line_irq), void(*frame_irq));

#endif // TS_H_INCLUDED
