
#include "Well.h"
#include "TS.h"
#include "string.h"
#include "Screen.h"

u8 well[well_height + 8][well_width + 8];

const u8 shape_pattern[7][4][4] =
{
    { // I
        { 0b0010, 0b0010, 0b0010, 0b0010 },
        { 0b0000, 0b1111, 0b0000, 0b0000 },
        { 0b0010, 0b0010, 0b0010, 0b0010 },
        { 0b0000, 0b1111, 0b0000, 0b0000 }
    },

    { // 0
        { 0b0011, 0b0011, 0b0000, 0b0000 },
        { 0b0000, 0b0011, 0b0011, 0b0000 },
        { 0b0000, 0b0110, 0b0110, 0b0000 },
        { 0b0110, 0b0110, 0b0000, 0b0000 }
    },

    { // S
        { 0b0010,
          0b0011,
          0b0001,
          0b0000 },
        { 0b0000,
          0b0011,
          0b0110,
          0b0000 },
        { 0b0010,
          0b0011,
          0b0001,
          0b0000 },
        { 0b0000,
          0b0011,
          0b0110,
          0b0000 }
    },

    { // Z
        { 0b0001,
          0b0011,
          0b0010,
          0b0000 },
        { 0b0000,
          0b0110,
          0b0011,
          0b0000 },
        { 0b0001,
          0b0011,
          0b0010,
          0b0000 },
        { 0b0000,
          0b0110,
          0b0011,
          0b0000 }
    },

    { // L
        { 0b0010, 0b0010, 0b0011, 0b0000 },
        { 0b0000, 0b0111, 0b0100, 0b0000 },
        { 0b0110, 0b0010, 0b0010, 0b0000 },
        { 0b0001, 0b0111, 0b0000, 0b0000 }
    },

    { // J
        { 0b0010, 0b0010, 0b0110, 0b0000 },
        { 0b0100, 0b0111, 0b0000, 0b0000 },
        { 0b0011, 0b0010, 0b0010, 0b0000 },
        { 0b0000, 0b0111, 0b0001, 0b0000 }
    },

    { // T
        { 0b0000, 0b0111, 0b0010, 0b0000 },
        { 0b0010, 0b0110, 0b0010, 0b0000 },
        { 0b0010, 0b0111, 0b0000, 0b0000 },
        { 0b0010, 0b0011, 0b0010, 0b0000 }
    }
};

const u16 T[8] =
{
    TILE_INFO(0,0,3,0),
    TILE_INFO(0,0,3,1),
    TILE_INFO(0,0,3,2),
    TILE_INFO(0,0,3,3),
    TILE_INFO(0,0,3,4),
    TILE_INFO(0,0,3,5),
    TILE_INFO(0,0,3,6),
    TILE_INFO(0,0,3,7)
};

void WellClear()
{
    u8 x,y;

    memset(&well, 0, sizeof(well));

    for(y=0;y<well_height+1;y++)
    {
        well[y+4][3] = 7;
        well[y+4][well_width+4] = 7;
    }

    for(x=0;x<well_width;x++)
        well[well_height+4][x+4] = 7;
}

u8 IsCollision(t_shape_pos *pos)
{
    u8 x,y;
    static u8 c;
    const u8* img = &shape_pattern[pos->shape][pos->rotate][0];
    u8* well_row = &well[pos->y][pos->x];

    for(y=0;y<4;y++)
    {
        c = img[y];
        for(x=0;x<4;x++)
        {
            if ((c & 0x08) && *well_row)
                return 1;
            c <<=1;
            well_row++;
        }
        well_row += well_width + 8 - 4;
    }

    return 0;
}

void DrawShape(t_shape_pos *pos, u8 color)
{
    u8 x,y;
    u8 c;
    const u8* img = &shape_pattern[pos->shape][pos->rotate][0];
    u8* well_row = &well[pos->y][pos->x];

    for(y=0;y<4;y++)
    {
        c = img[y];
        for(x=0;x<4;x++)
        {
            if (c & 0x08) *well_row = color;
            c <<=1;
            well_row++;
        }
        well_row += well_width + 8 - 4;
    }
}

void ShowWell()
{
    u8 x,y;
    u8 *well_row;
    u16 *tilemap_row;

    for(y=0; y<well_height; y++)
    {
        well_row = &well[y+4][4];
        tilemap_row = &tilemap[pad_top+y][1][pad_left+1];
        for(x=0; x<well_width;x++)
        {
            *tilemap_row = T[*well_row];
            tilemap_row++;
            well_row++;
        }
    }
}

